#include "gpio.h"
#include <QFile>

#define GPIO_PATH	"/sys/class/gpio/"
#define GPIO_EXPORT	GPIO_PATH "export"
#define GPIO_UNEXPORT	GPIO_PATH "unexport"
#define GPIO_DIR(nr)	GPIO_PATH "gpio" + QString::number(nr).toAscii()
#define GPIO_VALUE(nr)	GPIO_DIR(nr) + "/value"
#define GPIO_DIRECTION(nr)	GPIO_DIR(nr) + "/direction"
#define GPIO_DIRECTION_IN	"in"
#define GPIO_DIRECTION_OUT	"out"


gpio::gpio(int gpionumber)
{
	number = gpionumber;

	if(QFile::exists(GPIO_DIR(number)))
		openStatus = true;
	else
		openStatus = false;

}

gpio::~gpio()
{

}

bool gpio::isOpen()
{
	return openStatus;
}


bool gpio::open()
{
	if(!openStatus)
	{
		// Provide access to GPIO<number> by writing it's number to /sys/class/gpio/export
		// We skip this step if the directory /sys/class/gpio/gpio<number> already
		// exists.
		if(!QFile::exists(GPIO_DIR(number)))
		{

			// We first instantiate a QFile-Object to access the file 'export'
			QFile exp(GPIO_EXPORT);

			// Then we open it in WriteOnly-Mode
			if(!exp.open(QIODevice::WriteOnly))
				return false;

			/* Now we write the GPIO-number into the file.
			   It's essential to call the flush()-command after
			   the write()-command, because otherwise there is
			   no 100% guaranty that the number is written to the file.
			 */
			exp.write(QString::number(number).toAscii());
			exp.flush();

			// 2x is safer
			exp.write(QString::number(number).toAscii());
			exp.flush();

			if(QFile::exists(GPIO_DIR(number)))
			{
				openStatus = true;
				return true;
			}else
				return false;

		} else
		{
			openStatus = true;
			// GPIO is already open
			return true;
		}


	}else
		// Already open
		return true;

	//return true;
}


bool gpio::close()
{
	if(openStatus)
	{
		// If the function goes this path, we want to disconnect (unexport) the GPIO.
		// This is done by writing the GPIO's number '249' into the file 'unexport'
		// which is located in '/sys/class/gpio/'.
		QFile disconnect(GPIO_UNEXPORT);
		if(!disconnect.open(QIODevice::WriteOnly))
			return false;

		disconnect.write(QString::number(number).toAscii());
		disconnect.flush();

		if(!QFile::exists(GPIO_DIR(number)))
		{
			openStatus = false;
			return true;
		}else
			return false;

	}else
		//Already closed
		return true;
}


bool gpio::setDirection(GPIODirection direction)
{
	if(openStatus)
	{

		QFile connection(GPIO_DIRECTION(number));

		if(!connection.open(QIODevice::WriteOnly))
			return false;

		switch(direction)
		{
			case gpio::Input:
				connection.write(GPIO_DIRECTION_IN);
				connection.flush();
				break;

			case gpio::Output:
				connection.write(GPIO_DIRECTION_OUT);
				connection.flush();
				break;

			case NotOpen: return false;
				      break;

			default: return false;
		}

		return true;
	}else
		return false;
}


int gpio::getDirection()
{
	if(openStatus)
	{
		QFile connection(GPIO_DIRECTION(number));

		if(!connection.open(QIODevice::ReadOnly))
			return false;

		if(connection.readLine().startsWith(GPIO_DIRECTION_OUT))
			return gpio::Output;
		else
			return gpio::Input;

	}else
		return gpio::NotOpen;
}


bool gpio::setValue(GPIOValue value)
{
	if(openStatus)
	{
		QFile connection(GPIO_VALUE(number));

		if(!connection.open(QIODevice::WriteOnly))
			return false;

		if(value == gpio::High)
			connection.write("1");
		else if(value == gpio::Low)
		{
			connection.write("0");
		}else
			return false;

	}else
		return false;

	return true;
}


int gpio::getValue()
{
	if(openStatus)
	{
		QFile connection(GPIO_VALUE(number));
		if(!connection.open(QIODevice::ReadOnly))
			return gpio::NoValue;

		/*  The file 'value' should only consist of
		    one character, so it's sufficient to read
		    1 character only.
		 */
		if(connection.readLine()[0] == '1')
		{
			return gpio::High;
		}
		else{
			return gpio::Low;
		}

	}else
		return gpio::NoValue;
}

int gpio::getNumber()
{
	return number;
}
