#ifndef GPIO_H
#define GPIO_H

/**
  This class provides access to an IO.
  It implements all necessary functions to perform the following steps:
  - open/close connection to an IO
  - read from/write to an IO
  - get/set the IO's direction (input/output)
  How it works:
  All GPIOs can be found in the directory '/sys/class/gpio', but they're
  combined to so-called 'GPIOchips'. To make one GPIO accessible we need
  to write the appropriate GPIO number into the file called 'export',
  which is within the directory '/sys/class/gpio'. This will create a new
  directory (if it didn't already exist) 'GPIO<number>'. Within this new
  directory there are 2 important files, 'direction' and 'value'. 'direction'
  holds the information wether the GPIO should be an Input or an Output. We
  can write 'in' or 'out' into this file to change the setting. 'value'
  holds the state of our selected GPIO, which can be '0' or '1' (digital
  ports). When declared as an Input we can get the current state by reading
  from 'value'. When used as an Output we can write '0' or '1' to this file
  to set the GPIO's current state to High('1') or Low('0'). If the access
  to a GPIO is not needed anymore, we can unlink it by writing the GPIO's
  number to the file 'unexport' within the directory '/sys/class/gpio/',
  then the directory 'GPIO<number>' will disappear.
 */
class gpio
{

	public:
		/**
		  Constructor.
		 */
		gpio(int gpionumber);

		/**
		  Destructor
		 */
		~gpio();

		/**
		  Enum for IO - directions.
		 */
		enum GPIODirection {
			NotOpen,    /**< IO not open */
			Input, /**< Direction: input */
			Output /**< Direction: output */
		};

		/**
		  Enum for IO - values.
		 */
		enum GPIOValue {
			Low, /**< Value: Low / '0' */
			High, /**< Value: High / '1' */
			NoValue /**< no value -> IO not open */
		};

		/**
		  Returns if the IO is connected. If the IO is
		  open, it returns 'true', otherwise it returns 'false'.
		  @returns 'openStatus', the actual state of the connection to the IO
		  @see open()
		  @see close()
		 */
		bool isOpen();

		/**
		  Opens the connection to the IO.
		  It will return 'true' or 'false', whether the IO is now
		  open or not. It also returns 'true' if the IO is already
		  open.
		  @returns the new state of the IO
		  @see isOpen()
		  @see close()
		 */
		bool open();

		/**
		  Closes the connection to the IO.
		  It will return 'true' or 'false', whether the connection
		  was closed successfully or not. It also returns 'true' if
		  the connection was already closed.
		  @returns 'true' if closed successfully, 'false' if not
		  @see open()
		  @see isOpen()
		 */
		bool close();

		/**
		  Sets the IO-direction.
		  Possible directions are 'input' or 'output'.
		  @returns 'true' if set successfully, 'false' if not
		  @see getDirection()
		  @see setValue()
		  @see getValue()
		 */
		bool setDirection(GPIODirection direction);

		/**
		  Returns the IO-direction.
		  Possibl values can be 'input', 'output' or 'NotOpen'.
		  @returns 'true' if set successfully, 'false' if not
		  @see setDirection()
		  @see setValue()
		  @see getValue()
		 */
		int getDirection();

		/**
		  Sets the IO's value.
		  This only affects the IO, if it's declared as an output.
		  @param value the new value
		  @see getValue()
		  @see setDirection()
		  @see getDirection()
		 */
		bool setValue(GPIOValue value);

		/**
		  Returns the IO's value.
		  This will work whether the IO is declared as an input or output.
		  @param value the new value
		  @see setValue()
		  @see setDirection()
		  @see getDirection()
		 */
		int getValue();


		/**
		  Returns the IO - number.
		  @returns IO - number
		  @see getValue()
		  @see getDirection()
		 */
		int getNumber();

	protected:


	private:
		bool openStatus; /**< status of the connection to the IO */
		int number;/**< IO - number */

};

#endif // GPIO_H
