#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFile>
#include <QKeyEvent>
#include <QDebug>

#include "settingsdialog.h"
#include "../helpdialog.h"
#include "../touchbuttons.h"


MainWindow::MainWindow(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::MainWindow)
{
	ui->setupUi(this);

	// Maximize window to fullscreen
	this->showFullScreen();

	// Set Backgroundcolor: White
	QPalette pal = this->palette();
	pal.setColor(QPalette::Window, Qt::white);
	this->setPalette(pal);


	// Connect signals and slots
	connect(ui->toggleButton, SIGNAL(clicked()), this, SLOT(toggleButtonClicked()));
	connect(ui->onButton, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
	connect(ui->offButton, SIGNAL(clicked()), this, SLOT(offButtonClicked()));
	connect(ui->clearButton, SIGNAL(clicked()), this, SLOT(clearEdit()));

	connect(ui->backButton, SIGNAL(clicked()), this, SLOT(backButtonClicked()));
	connect(ui->settingsButton, SIGNAL(clicked()), this, SLOT(settingsButtonClicked()));
	connect(ui->helpButton, SIGNAL(clicked()), this, SLOT(helpButtonClicked()));

	// Setup GPIO information
	gpionumber = 249;
	ui->gpioNameLabel->setText("Relay (249)");
	currentgpio = new gpio(249);

	// Setup connection to the relay
	if(!currentgpio->isOpen())
	{
		if(currentgpio->open())
			ui->outputEdit->append("\"249\" written to /sys/class/gpio/export");

	}
	if(currentgpio->isOpen())
	{
		currentgpio->setDirection(gpio::Output);
		ui->outputEdit->append("\"out\" written to /sys/class/gpio/gpio249/direction");
	}

}

void MainWindow::keyPressEvent(QKeyEvent *event)
{
	//qDebug()<<event->key()<<" ("<<event->text()<<") pressed. - MainWindow"<<endl;

	switch(event->key())
	{
		case touchbuttons::Key_Touch_Left:
			ui->settingsButton->setDown(true);
			break;
		case touchbuttons::Key_Touch_Center:
			ui->backButton->setDown(true);
			break;
		case touchbuttons::Key_Touch_Right:
			ui->helpButton->setDown(true);
			break;
		default:
			break;
	}

}

void MainWindow::keyReleaseEvent(QKeyEvent *event)
{
	//qDebug()<<event->key()<<" ("<<event->text()<<") released. - MainWindow"<<endl;

	switch(event->key())
	{
		case touchbuttons::Key_Touch_Left:
			if(ui->settingsButton->isDown())
			{
				ui->settingsButton->setDown(false);
				ui->settingsButton->click();
			}else{
				ui->settingsButton->setDown(false);
			}        
			break;
		case touchbuttons::Key_Touch_Center:
			if(ui->backButton->isDown())
			{
				ui->backButton->setDown(false);
				ui->backButton->click();
			}else{
				ui->backButton->setDown(false);
			}
			break;
		case touchbuttons::Key_Touch_Right:
			if(ui->helpButton->isDown())
			{
				ui->helpButton->setDown(false);
				ui->helpButton->click();
			}else {
				ui->helpButton->setDown(false);
			}
			break;
		default:
			break;
	}

}

void MainWindow::settingsButtonClicked()
{

	int newgpionumber = 0;

	// Open Settings-Dialog to get new number
	SettingsDialog dialog;
	dialog.setNumber(gpionumber);
	if(dialog.exec() == QDialog::Accepted)
	{
		newgpionumber = dialog.getNumber();
	} else
	{
		return;
	}

	// Return from Settings-Dialog

	if(newgpionumber != gpionumber)
	{
		if(currentgpio->isOpen())
		{
			// Close current GPIO
			currentgpio->close();
			ui->outputEdit->append("\"" + QString::number(gpionumber).toAscii() +"\" written to /sys/class/gpio/unexport");
		}

		gpionumber = newgpionumber;

		// Open new GPIO
		currentgpio = new gpio(newgpionumber);
		if(!currentgpio->isOpen())
		{
			if(currentgpio->open())
			{
				ui->outputEdit->append("\"" + QString::number(newgpionumber).toAscii() +"\" written to /sys/class/gpio/export");
			}else{
				ui->outputEdit->append("Can't access GPO \"" + QString::number(newgpionumber).toAscii() +"\" at this moment. (Busy / in use / protected)");
				return;
			}
		}
		currentgpio->setDirection(gpio::Output);
		ui->outputEdit->append("\"out\" written to /sys/class/gpio/gpio" + QString::number(newgpionumber).toAscii() + "/direction");

		//Update UI
		switch (newgpionumber)
		{
			case 249: ui->gpioNameLabel->setText("Relay (249)");
				  break;
			case 250: ui->gpioNameLabel->setText("Optocoupler (250)");
				  break;
			default: ui->gpioNameLabel->setText(QString::number(newgpionumber));

		}

		ui->statusLabel->setText("-");

	}else
	{
		ui->outputEdit->append("Nothing changed.");
	}

}


void MainWindow::clearEdit()
{
	ui->outputEdit->clear();
}



/*
   This function is called when the 'Toggle'-Button was
   pushed. It toggles the state of the Relay from ON to OFF
   / OFF to ON. The new state of the relay depends on the
   actual state, which is retrieved by calling the function
   getRelayStatus().
 */
void MainWindow::toggleButtonClicked()
{
	if(currentgpio->isOpen())
	{
		switch(currentgpio->getValue())
		{
			case gpio::High:
				ui->outputEdit->append("\"1\" read from /sys/class/gpio/gpio" + QString::number(gpionumber).toAscii() + "/value");
				if(currentgpio->setValue(gpio::Low))
				{
					ui->statusLabel->setText("OFF");
					ui->outputEdit->append("\"0\" written to /sys/class/gpio/gpio" + QString::number(gpionumber).toAscii() + "/value");
				}
				break;
			case gpio::Low:
				ui->outputEdit->append("\"0\" read from /sys/class/gpio/gpio" + QString::number(gpionumber).toAscii() + "/value");
				if(currentgpio->setValue(gpio::High))
				{
					ui->statusLabel->setText("ON");
					ui->outputEdit->append("\"1\" written to /sys/class/gpio/gpio" + QString::number(gpionumber).toAscii() + "/value");
				}
				break;
			case gpio::NoValue:
				ui->statusLabel->setText("-");
				ui->outputEdit->append("Can't access GPO \"" + QString::number(gpionumber).toAscii() +"\" at this moment. (Busy / in use / protected)");
		}



	}else
	{
		ui->outputEdit->append("Can't access GPO \"" + QString::number(gpionumber).toAscii() +"\" at this moment. (Busy / in use / protected)");
	}
}


void MainWindow::backButtonClicked()
{
	close();
}

void MainWindow::onButtonClicked()
{
	if(currentgpio->isOpen())
	{
		currentgpio->setValue(gpio::High);
		ui->statusLabel->setText("ON");
		ui->outputEdit->append("\"1\" written to /sys/class/gpio/gpio" + QString::number(gpionumber).toAscii() + "/value");
	}else
	{
		ui->outputEdit->append("Can't access GPO \"" + QString::number(gpionumber).toAscii() +"\" at this moment. (Busy / in use / protected)");
	}

}

void MainWindow::offButtonClicked()
{
	if(currentgpio->isOpen())
	{
		currentgpio->setValue(gpio::Low);
		ui->statusLabel->setText("OFF");
		ui->outputEdit->append("\"0\" written to /sys/class/gpio/gpio" + QString::number(gpionumber).toAscii() + "/value");
	}else
	{
		ui->outputEdit->append("Can't access GPO \"" + QString::number(gpionumber).toAscii() +"\" at this moment. (Busy / in use / protected)");
	}
}

void MainWindow::helpButtonClicked()
{
	HelpDialog dialog;
	dialog.loadURL("qrc:///files/IODemo_Help.html");
	dialog.exec();

}


/*
   ~MainWindow() is the destructor of this class 'MainWindow'. It's
   called if an instance of this class is not needed anymore,
   e.g. if the function close() was called. Within this function
   we disconnect/unexport the GPIO249 that we used by calling
   setupRelayConnection(false).
 */
MainWindow::~MainWindow()
{

	currentgpio->close();

	delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
	QMainWindow::changeEvent(e);
	switch (e->type()) {
		case QEvent::LanguageChange:
			ui->retranslateUi(this);
			break;
		default:
			break;
	}
}
